/*
 * This file is part of libcreds.
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * @file creds.h
 *
 * Credentials handling library.
 *
 * The current credentials information supported by
 * this library, includes following:
 *
 * - user id (UID)
 * - group id (GID)
 * - capabilities
 * - supplementary groups
 *
 */
#ifndef _CREDS_H
#define _CREDS_H

#ifdef	__cplusplus
extern "C" {
#endif

/*!
 * Opaque credentials handle.
 *
 * The internal structure is defined by the implementation.
 */
typedef struct _creds_struct *creds_t;

/*!
 * Types of credentials information
 */
typedef enum {
	CREDS_BAD = -1,	/* Indicates error */
	CREDS_UID,	/* User Identifier -- uid_t */
	CREDS_GID,	/* Group Identifier -- gid_t */
	CREDS_CAP,	/* Capability Number -- cap_value_t */
	CREDS_FIXED	/* The number of predefined CREDS Identifiers */
	/* Dynamically assigned identifiers start from CREDS_FIXED ... */
} creds_type_t;

/*!
 * Values of the credentials
 */
typedef long creds_value_t;

/*!
 * Read peer credentials from a socket.
 *
 * @param socket The socket
 * @return a handle for the credentials
 *
 * The handle must be freed with creds_free.
 *
 * A null return indicates failed operation.
 */
creds_t creds_getpeer(int socket);

/*!
 * Read credentials from a task
 *
 * @param pid The process id
 * @return a handle for the credentials
 *
 * The handle must be freed with creds_free.
 *
 * A null return indicates failed operation.
 */
creds_t creds_gettask(pid_t pid);

/*!
 * Free the memory allocated to credentials information.
 *
 * @param creds The credentials handle.
 *
 * It is safe to call this even with NULL handle.
 */
void creds_free(creds_t creds);

/*!
 * Modify credentials of current process.
 *
 * The operation can only succeed, if the current
 * process has enough capabilities to do the change.
 *
 * @param The handle for credentials to be applied.
 */
int creds_set(const creds_t creds);

/*!
 * Universal conversion function.
 *
 * The credential name is assumed to include "namespace"
 *
 *   - uid::user_name
 *   - gid::group_name
 *   - cap::capability_name
 *
 * @param credential The name
 * @return credential type of CREDS_BAD, if conversion failed.
 *
 * The implementation can decide how deal with omitted namespace,
 * or a namespace not listed above.
 */
long creds_str2creds(const char *credential, creds_value_t *value);

/*!
 * Iterate over all credentials values.
 *
 * @param creds The credentials handle.
 * @param index The index of the credential [0..N]
 * @param value The returned credentials value
 * @return type of the credential value, or CREDS_BAD when no more.
 */
creds_type_t creds_list(const creds_t creds, int index, creds_value_t *value);

/*!
 * Test presence of a credential.
 *
 * @param creds The credentials handle
 * @param type The type of credentials to test
 * @param value The specific value to test
 * @return 1, if credential is present, and 0 otherwise.
 */
int creds_have_p(const creds_t creds, creds_type_t type, creds_value_t value);

#ifdef	__cplusplus
}
#endif
#endif
