/*
 * This file is part of libcreds.
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * @file creds.c
 *
 * This is proof-of-concept/reference implementation of the credentials
 * library (libcreds). This implementation does not require any kernel
 * modifications, and only relies on /proc/<pid>/status and other current
 * kernel (2.6.x) features.
 *
 * Because "/proc/<pid>/status" only returns at most NGROUPS_SMALL (= 32)
 * supplementary groups, this implementation will not return all groups
 * in "creds_gettask", if the task groups exceeds NGROUPS_SMALL.
 *
 * In future, this implementation should be replaced to use some more
 * efficient and complete way of getting the credentials from the
 * kernel -- a new syscall needed?
 *
 * *NOTE* This implementation requires the availability of libcap2
 * or later (libcap1 is not enough!).
 *
 * *NOTE* This implementation code is Linux specific.
 */

#define _ISOC99_SOURCE /* ..to get isblank from ctypes.h */
#define _GNU_SOURCE /* ..to get struct ucred from sys/socket.h */

#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/prctl.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <ctype.h>
#include <pwd.h>
#include <grp.h>
#include <sys/creds.h>
#include <sys/capability.h>

/* the "/proc/<pid>/status" only includes NGROUPS_SMALL number of
   supplementary groups. This value is not exported from kernel?
   Assume it is 32 (based on 2.6.24 kernel code), and define it here..
*/
#define NGROUPS_SMALL 32

struct _creds_struct {
	uid_t uid;
	gid_t gid;
	cap_t cap;
	unsigned n_groups;
	gid_t *groups;
};

enum {
	STATUS_UID,
	STATUS_GID,
	STATUS_GROUPS,
	STATUS_CAPPRM,
	STATUS_CAPINH,
	STATUS_CAPEFF,
	STATUS_MAX
};


struct stream_buf {
	int fd;
	int eol;
	int head;
	int tail;
	char data[200];
};

static const struct _creds_struct empty_creds = {
	.uid = CREDS_BAD,
	.gid = CREDS_BAD,
};

#define STRING(s) { s, sizeof(s)-1 }

/* Labels for extacting stuff from proc status */
static const struct {
	const char *prefix;
	size_t len;
} grok[STATUS_MAX] = {
	[STATUS_UID] = STRING("Uid:"),
	[STATUS_GID] = STRING("Gid:"),
	[STATUS_GROUPS] = STRING("Groups:"),
	[STATUS_CAPINH] = STRING("CapInh:"),
	[STATUS_CAPPRM] = STRING("CapPrm:"),
	[STATUS_CAPEFF] = STRING("CapEff:"),
};

/* Prefixes of supported credentials types used
 * by the string to value conversion.
 */
static const struct {
	const char *const prefix;
	size_t len;
} creds_fixed_types[CREDS_FIXED] = {
	[CREDS_UID] = STRING("uid::"),
	[CREDS_GID] = STRING("gid::"),
	[CREDS_CAP] = STRING("cap::"),
};

static creds_t creds_init(void)
{
	creds_t creds = (creds_t)malloc(sizeof(struct _creds_struct));
	if (creds)
		*creds = empty_creds;
	return creds;
}

creds_t creds_getpeer(int fd)
{
	struct ucred cr;
	size_t cr_len = sizeof(cr);
	if (getsockopt (fd, SOL_SOCKET, SO_PEERCRED, &cr, &cr_len) == 0 &&
	    cr_len == sizeof(cr))
		return creds_gettask(cr.pid);
	return NULL;
}


static int pull(struct stream_buf *buf, int req_len)
{
	int len;
	while ((len = buf->tail - buf->head) < req_len) {
		int room;
		if (buf->head > 0) {
			if (len > 0)
				memmove(buf->data, buf->data + buf->head, len);
			buf->head = 0;
			buf->tail = len;
		}
		if (buf->fd < 0)
			return 0; /* No data */

		room = sizeof(buf->data) - buf->tail;
		len = read(buf->fd, buf->data + buf->tail, room);
		if (len <= 0) {
			/* No more data available */
			close(buf->fd);
			buf->fd = -1;
			return 0;
		}
		buf->tail += len;
	}
	return len;
}

static void skip_line(struct stream_buf *buf)
{
	int len;
	while ((len = pull(buf, 1)) > 0) {
		char *s = buf->data + buf->head;
		char *nl = memchr(s, '\n', len);
		if (nl) {
			len = nl - s;
			buf->eol = 0;
			buf->head += len + 1;
			return;
		}
		buf->head += len;
	}
	/* ...missing new line at end of file? */
	buf->eol = 1;
}

static void skip_blank(struct stream_buf *buf)
{
	while (pull(buf, 1)) {
		const int c = buf->data[buf->head];
		if (c == '\n') {
			buf->eol = 1;
			break;
		}
		if (!isblank(c))
			break;
		buf->head += 1;
	}
}

static int get_next_int(struct stream_buf *buf)
{
	int good = 0;
	int value = 0;
	
	if (buf->eol)
		return CREDS_BAD;
	
	skip_blank(buf);
	
	/* Pull decimal digits */
	while (pull(buf, 1)) {
		const int c = buf->data[buf->head];
		if (!isdigit(c))
			break;
		value = value * 10 + c - '0';
		buf->head += 1;
		good = 1;
	}
	return good ? value : CREDS_BAD;
}

static int get_cap_bits(struct stream_buf *buf, creds_t creds, cap_flag_t flag)
{
	cap_value_t cap = 0;
	int count = 0;
	char *ptr;
	
	skip_blank(buf);
	if (buf->eol) {
		errno = EINVAL;
		return -1; /* There should always be something! */
	}
	if (creds->cap == NULL && (creds->cap = cap_init()) == NULL)
		return -1;

	/* The caps is a hex string of undefined length, although currently 16, of
	   which only 32 lowest bits can be set. Need to parse in reverse, to find the
	   capability bit numbers. This should work with any number of capabilities.
	   [Code does not rely on any specific integer size for the hex string,
	   because the hex string is not converted to single number...]
	*/

	/* Find the length of the hex string */
	while (pull(buf, count + 1)) {
		const int c = buf->data[buf->head + count];
		/* Just allow blanks within hex digits, in case someone decides
		   to group the hex digits somehow... */
		if (!isxdigit(c) && !isblank(c))
			break; /* Not hex nor blank! */
		++count;
	}

	ptr = buf->data + buf->head + count;
	buf->head += count;
	while (--count >= 0) {
		int i = 4;
		int c = *--ptr;
		unsigned int bits;
		if (isblank(c))
			continue; /* Ignore blanks */
		c = tolower(c);
		bits = isdigit(c) ? c - '0' : 10 + c - 'a';
		while (--i >= 0) {
			if (bits & 1)
				cap_set_flag(creds->cap, flag, 1, &cap, 1);
			bits >>= 1;
			cap++;
		}
	}
	return 0; /* Success */
}

static creds_t get_creds_from_proc(pid_t pid)
{
	struct stream_buf buf;
	creds_t creds;

	buf.head = 0;
	buf.tail = 0;
	buf.eol = 0;

	/* borrow the buf.data for the filename */
	if (pid)
		snprintf(buf.data, sizeof(buf.data), "/proc/%d/status", pid);
	else
		strncpy(buf.data, "/proc/self/status", sizeof(buf.data));
	buf.fd = open(buf.data, O_RDONLY);
	if (buf.fd < 0)
		return NULL;

	creds = creds_init();
	if (creds == NULL)
		goto out;

	while (pull(&buf, 1)) {
		int i;
		/* Determine the type of the line */
		for (i = 0; i < STATUS_MAX; ++i) {
			if (pull(&buf, grok[i].len) && memcmp(grok[i].prefix, buf.data + buf.head, grok[i].len) == 0) {
				// Matched the keyword...
				buf.head += grok[i].len;
				break;
			}
		}
		
		/* Handle the line content */
		switch (i) {
		case STATUS_UID:
			creds->uid = get_next_int(&buf); /* Skip uid */
			creds->uid = get_next_int(&buf); /* Get effective uid */
			break;
		case STATUS_GID:
			creds->gid = get_next_int(&buf); /* Skip gid */
			creds->gid = get_next_int(&buf); /* Get effective gid */
			break;
		case STATUS_GROUPS:
			while (creds->n_groups < NGROUPS_SMALL) {
				skip_blank(&buf);
				if (buf.eol)
					break;
				if (creds->groups == NULL) {
					creds->groups = (gid_t *)malloc(NGROUPS_SMALL * sizeof(gid_t));
					if (creds->groups == NULL)
						goto out;
				}
				creds->groups[creds->n_groups++] = get_next_int(&buf);
			}
			break;
		case STATUS_CAPPRM:
			if (get_cap_bits(&buf, creds, CAP_PERMITTED))
				goto out;
			break;
		case STATUS_CAPINH:
			if (get_cap_bits(&buf, creds, CAP_INHERITABLE))
				goto out;
			break;
		case STATUS_CAPEFF:
			if (get_cap_bits(&buf, creds, CAP_EFFECTIVE))
				goto out;
			break;
		default:
			break;
		}
		/* Skip over to the beginning of the next line */
		skip_line(&buf);
	}
	if (buf.fd >= 0)
		close(buf.fd);
	return creds;
	
out:
	/* Detetected an error, do not silently return incomplete data! */
	creds_free(creds);
	
	if (buf.fd >= 0)
		close(buf.fd);
	return NULL;
}


creds_t creds_gettask(pid_t pid)
{
	creds_t creds;
	int n = NGROUPS_SMALL;

	if (pid)
		return get_creds_from_proc(pid);

	/* Can get credentials of the current process without procfs */

	creds = creds_init();
	if (creds == NULL)
		goto out;

	creds->uid = geteuid(); /* Note: effective uid! */
	creds->gid = getegid(); /* Note: effective gid! */

	/* For caps too, we really are interested only on effective? */
	creds->cap = cap_get_proc();

	do {
		creds->groups = (gid_t *)realloc(creds->groups, n * sizeof(gid_t));
		if (creds->groups == NULL)
			goto out;
		creds->n_groups = getgroups(n, creds->groups);
		n = n * 2;
	} while (creds->n_groups < 0);
	
	return creds;
	
out:
	/* Detetected an error, do not silently return incomplete data! */
	creds_free(creds);
	return NULL;
}

void creds_free(creds_t creds)
{
	if (creds) {
		if (creds->groups)
			free(creds->groups);
		if (creds->cap)
			cap_free(creds->cap);
		free(creds);
	}
}

int creds_set(const creds_t creds)
{
	if (! creds) {
		errno = EINVAL;
		return -1;
	}
	if (setgroups(creds->n_groups, creds->groups))
		return -1;
	if (creds->gid != CREDS_BAD && setgid(creds->gid))
		return -1;

	if (creds->uid != CREDS_BAD) {
		int my_errno = 0;
		const int keepcaps = prctl(PR_GET_KEEPCAPS, 0, 0, 0, 0);
		if (keepcaps == 0 && prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0)) {
			/* Could not set keepcaps, but ignore it for now */
		}
		
		if (setuid(creds->uid))
			my_errno = errno;

		//Restore original prctl KEEPCAPS state!
		if (keepcaps == 0 && prctl(PR_SET_KEEPCAPS, 0, 0, 0, 0)) {
			/* Failed to restore original keepcaps, but ignore it for now */
		}
		if (my_errno) {
			/* Ignore possible error from prctl */
			errno = my_errno;
			return -1;
		}
	}
	if (creds->cap)
		return cap_set_proc(creds->cap);
	else {
		int res = -1;
		cap_t caps = cap_init();
		caps = cap_init();
		if (caps) {
			res = cap_set_proc(caps);
			cap_free(caps);
		}
		return res;
	}
}

static int numeric_p(const char *str, long *value)
{
	/* Note: this internal help function assumes
	   that both str and value are not NULL, and
	   that str is not empty! */

	char *endptr;
	int saved = errno;
	int ret = 1;

	errno = 0;
	*value = strtol(str, &endptr, 10);
	if (errno || *endptr)
		ret = 0; /* numeric conversion failed */
	errno = saved;
	return ret;
}

static long creds_str2uid(const char *user)
{
	int retry;
	char *buf = NULL;
	size_t buflen = 1024;
	uid_t uid = CREDS_BAD;
	long nbr;

	if (!user || !*user)
		return uid;

	if (numeric_p(user, &nbr))
		return nbr;

	for (retry = 0; retry < 5; ++retry) {
		int res;
		struct passwd p;
		struct passwd *pptr = NULL;
		char *newbuf = (char *)realloc(buf, buflen);

		if (!newbuf)
			break;
		buf = newbuf;
		res = getpwnam_r(user, &p, buf, buflen, &pptr);
		if (res == 0 && pptr == &p) {
			uid = p.pw_uid;
			break; /* Converted user to uid successfully */
		}
		if (res != ERANGE);
			break;
		buflen *= 2;
	}
	if (buf)
		free(buf);
	return uid;
}

static long creds_str2gid(const char *group)
{
	int retry;
	char *buf = NULL;
	size_t buflen = 1024;
	gid_t gid = CREDS_BAD;
	long nbr;

	if (!group || !*group)
		return gid;

	if (numeric_p(group, &nbr))
		return nbr;

	for (retry = 0; retry < 5; ++retry) {
		int res;
		struct group g;
		struct group *gptr = NULL;
		char *newbuf = (char *)realloc(buf, buflen);

		if (!newbuf)
			break;
		buf = newbuf;
		res = getgrnam_r(group, &g, buf, buflen, &gptr);
		if (res == 0 && gptr == &g) {
			gid = g.gr_gid;
			break; /* Converted group to gid successfully */
		}
		if (res != ERANGE);
			break;
		buflen *= 2;
	}
	if (buf)
		free(buf);
	return gid;
}

static long creds_str2capability(const char *capability)
{
	cap_value_t cap = CREDS_BAD;
	long nbr;

	if (!capability || !*capability)
		return cap;

	if (numeric_p(capability, &nbr))
		return nbr;

	return cap_from_name(capability, &cap) ? CREDS_BAD : cap;
}

long creds_str2creds(const char *credential, creds_value_t *value)
{
	int len;
	int i;

	if (!value)
		return CREDS_BAD;
	*value = CREDS_BAD;
	if (!credential)
		return CREDS_BAD;

	len = strlen(credential);
	for (i = 0; i < sizeof(creds_fixed_types) / sizeof(creds_fixed_types[0]); ++i) {
		const size_t cmplen = creds_fixed_types[i].len;
		if (cmplen < len &&
		    memcmp(creds_fixed_types[i].prefix, credential, cmplen) == 0) {
			/* prefix matched */
			credential += cmplen;
			break;
		}
	}
	switch (i) {
	case CREDS_UID:
		*value = creds_str2uid(credential);
		break;
	case CREDS_GID:
		*value = creds_str2gid(credential);
		break;
	case CREDS_CAP:
		*value = creds_str2capability(credential);
		break;
	default:
		/* ...could test uid/gid/cap in some order? */
		i = CREDS_BAD;
		break;
	}
	return (*value == CREDS_BAD) ? CREDS_BAD : i;
}

creds_type_t creds_list(const creds_t creds, int index, creds_value_t *value)
{
	int i;
	cap_flag_value_t val;

	if (!creds || !value || index < 0)
		return CREDS_BAD;

	/* check uid slot */
	if (creds->uid != CREDS_BAD) {
		if (index == 0) {
			*value = creds->uid;
			return CREDS_UID;
		}
		--index;
	}

	/* check gid slot */
	if (creds->gid != CREDS_BAD) {
		if (index == 0) {
			*value = creds->gid;
			return CREDS_GID;
		}
		--index;
	}

	/* check supplementary groups */
	if (index < creds->n_groups) {
		*value = creds->groups[index];
		return CREDS_GID;
	}
	index -= creds->n_groups;

	/* check capabilities */

	/* ...there appears to be no way to find max capability number,
	   thus just need to rely on cap_get_flag to return -1, when
	   we go over the max ... */
	for (i = 0; cap_get_flag(creds->cap, i, CAP_EFFECTIVE, &val) == 0; ++i) {
		if (val == CAP_SET) {
			if (index == 0) {
				*value = i;
				return CREDS_CAP;
			}
			--index;
		}
	}
	return CREDS_BAD;
}

int creds_have_p(const creds_t creds, creds_type_t type, creds_value_t value)
{
	int i;
	cap_flag_value_t val;

	if (! creds)
		return 0;
	switch (type) {
	case CREDS_CAP:
		if (cap_get_flag(creds->cap, value, CAP_EFFECTIVE, &val))
			return 0;
		return val == CAP_SET;
	case CREDS_UID:
		return creds->uid == value;
	case CREDS_GID:
		/* test specific gid (includes supplementary groups) */
		if (creds->gid == value)
			return 1;
		for (i = 0; i < creds->n_groups; ++i)
			if (creds->groups[i] == value)
				return 1;
		break;
	default:
		break;
	}
	return 0;
}
